package djmil.cordacheckers.gameboard;

import java.util.Map;
import java.util.UUID;

import djmil.cordacheckers.contracts.GameBoardCommand;
import djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil;
import djmil.cordacheckers.states.Counterparty.NotInvolved;
import djmil.cordacheckers.states.GameBoardState;
import djmil.cordacheckers.states.Piece;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;


// GameBoard from the player's point of view
public class GameBoardView {
    public final String opponentName;
    public final Piece.Color opponentColor;
    public final Boolean opponentMove;
    public final Integer moveNumber;
    public final Map<Integer, Piece> board;
    public final GameBoardCommand previousCommand;
    public final String message;
    public final UUID id;

    /*
     * GameStatus enum:
     * - YOUR_TURN
     * - WAIT_FOR_OPPONENT
     * - VICTORY
     * - YOU_LOOSE
     */

    // Serialisation service requires a default constructor
    public GameBoardView() {
        this.opponentName = null;
        this.opponentColor = null;
        this.opponentMove = null;
        this.moveNumber = null;
        this.board = null;
        this.previousCommand = null;
        this.message = null;
        this.id = null;
    }

    // A view from a perspective of a concrete player, on a ledger transaction that has 
    // produced new GameBoardState
    public GameBoardView(UtxoLedgerTransaction utxoGameBoard, MemberX500Name myName) throws NotInvolved {
        // TODO check command type: MOVE vs FINNISH | SURRENDER
        //                  SingleOutPut vs SingleInput
        this.previousCommand = UtxoLedgerTransactionUtil
            .getOptionalCommand(utxoGameBoard, GameBoardCommand.class)
            .orElseGet(() -> null); // there is no previous command for GameProposal.Accept case

        final GameBoardState stateGameBoard = UtxoLedgerTransactionUtil
            .getSingleOutputState(utxoGameBoard, GameBoardState.class);

        this.opponentName  = stateGameBoard.getCounterpartyName(myName).getCommonName();
        this.opponentColor = stateGameBoard.getCounterpartyColor(myName);

        this.opponentMove  = this.opponentColor == stateGameBoard.getMoveColor();
        this.moveNumber    = stateGameBoard.getMoveNumber();
        this.board   = stateGameBoard.getBoard();
        this.message = stateGameBoard.getMessage();
        this.id      = stateGameBoard.getId();
    }

}
